@@ -41,67 +41,6 @@ use Spp::Tools;
 # Optimizer Ast
 #######################
 
-sub opt_exprs {
-  my $atoms = shift;
-  my $atoms_opt = [];
-  for my $atom (values $atoms) {
-    push $atoms_opt, opt_atom($atom);
-  }
-  return add_exprs($atoms_opt);
-}
-
-sub opt_atom {
-  my $atom = shift;
-  return ['true'] if $atom eq '{}';
-  my ($name, $value) = @{$atom};
-  # see $name; exit;
-  if (is_perl_array($name)) {
-    my $opt_atoms = opt_atoms($atom);
-    return add_exprs($opt_atoms);
-  }
-  given ($name) {
-    when ('sym')     { return opt_sym($value)    }
-    when ('char')    { return opt_char($value)   }
-    when ('Str')     { return opt_str($value)    }
-    when ('Keyword') { return opt_str($value)    }
-    when ('String')  { return opt_string($value) }
-    when ('List')    { return opt_list($value)   }
-    when ('Array')   { return opt_array($value)  }
-    when ('Sarray')  { return opt_sarray($value) }
-    when ('Hash')    { return opt_hash($value)   }
-    when ('Pair')    { return opt_atoms($value)  }
-    when ('Rule')    { return opt_rule($value)   }
-    when ('Group')   { return opt_group($value)  }
-    when ('Strs')    { return opt_strs($value)   }
-    when ('Chclass') { return opt_chclass($value)}
-    when ('Action')  { return opt_atom($value)   }
-    when ('Alias')   { return opt_alias($value)  }
-    when ('chars')   { return $value             }
-    when ('int')     { return $value + 0         }
-    when ('any')     { return ['any']            }
-    when ('dot')     { return ['dot']            }
-    when ('ctoken')  { return ['ctoken', $value] }
-    when ('rtoken')  { return ['rtoken', $value] }
-    when ('gtoken')  { return ['gtoken', $value] }
-    when ('assert')  { return ['assert', $value] }
-    when ('cclass')  { return ['cclass', $value] }
-    # if String only have blank interlatation
-    when ('inter')   { return ''                 }
-    see $atom;
-    default { error("Unknown type to optimizer: $name") }
-  }
-}
-
-sub opt_sym {
-  my $name = shift;
-  given ($name) {
-    when ('nil')  { return ['nil'] }
-    when ('true')  { return ['true'] }
-    when ('false') { return ['false'] }
-    default { return ['sym', $name] }
-  }
-}
-
 sub opt_str_char {
   my $value = shift;
   my $char = tail($value);
@@ -113,73 +52,6 @@ sub opt_str_char {
   }
 }
 
-sub opt_char {
-  my $char = shift;
-  return opt_str_char($char);
-}
-
-sub opt_sarray {
-  my $ast = shift;
-  my ($strs, $str) = ([], '');
-  for my $node (values $ast) {
-    my ($name, $value) = @{ $node };
-    given ($name) {
-      when ('cstr') { $str = $str . $value }
-      when ('char') { $str = $str . opt_str_char($value) }
-      when ('blank') {
-        push $strs, $str if len($str) > 0;
-        $str = '';
-      }
-      default { error("Unknown Aarray node name: $name") }
-    }
-  }
-  push $strs, $str if len($str) > 0;
-  return ['array', $strs];
-}
-
-sub opt_str {
-  my $ast = shift;
-  return '' if $ast eq "''";
-  my $chars = [];
-  for my $node (values $ast) {
-    my ($name, $value) = @{$node};
-    if ($name eq 'char') {
-      push $chars, opt_str_char($value);
-    } else {
-      push $chars, $value;
-    }
-  }
-  return perl_join($chars);
-}
-
-# my $ast = [["dstr","str"]];
-# see opt_string($ast);
-sub opt_string {
-  my $ast = shift;
-  return '' if $ast eq '""';
-  my $nodes = [];
-  for my $node (values $ast) {
-    my ($name, $value) = @{$node};
-    given ($name) {
-      when ('char') {
-        my $char_str = opt_str_char($value);
-        push $nodes, $char_str;
-      }
-      when ('dstr') {
-        push $nodes, $value;
-      }
-      default {
-        push $nodes, opt_atom($node);
-      }
-    }
-  }
-  my $str_nodes = combin_str_nodes($nodes);
-  if (len($str_nodes) == 1 and is_str($nodes->[0]) ) {
-    return first($str_nodes);
-   }
-  return ['string', $str_nodes];
-}
-
 sub combin_str_nodes {
   my $nodes = shift;
   my $str_nodes = [];
@@ -191,7 +63,7 @@ sub combin_str_nodes {
     }
     else {
       if ($str_mode == 1) {
-        push $str_nodes, $str;
+        push $str_nodes, ['str', $str];
       }
       push $str_nodes, $node;
       $str = '';
@@ -199,122 +71,11 @@ sub combin_str_nodes {
     }
   }
   if ($str_mode == 1) {
-    push $str_nodes, $str;
+    push $str_nodes, ['str', $str];
   }
   return $str_nodes;
 }
 
-sub opt_hash {
-  my $hash = shift;
-  # hash would saved as array to contain keys value
-  return ['hash', []] if $hash eq '{}';
-  my $hash_opt = [];
-  for my $pair (values $hash) {
-    if (first($pair) eq 'Pair') {
-      my $pair_value = tail($pair);
-      my $opt_pair = opt_atoms($pair_value);
-      push $hash_opt, $opt_pair;
-    } else {
-      error("Hash key name not is <Pair>");
-    }
-  }
-  return ['hash', $hash_opt];
-}
-
-sub opt_atoms {
-  my $atoms = shift;
-  my $atoms_opt = [];
-  for my $atom (values $atoms) {
-    push $atoms_opt, opt_atom($atom);
-  }
-  return $atoms_opt;
-}
-
-sub opt_list {
-  my $atoms = shift;
-  return ['list', []] if $atoms eq '()';
-  my $atoms_opt = opt_atoms($atoms);
-  return ['list', $atoms_opt];
-}
-
-sub opt_array {
-  my $atoms = shift;
-  return ['array', []] if $atoms eq '[]';
-  my $atoms_opt = opt_atoms($atoms);
-  return ['array', $atoms_opt];
-}
-
-sub opt_rule {
-  my $atoms = shift;
-  return ['true'] if $atoms eq ':{}';
-  my $rule_value = opt_token($atoms);
-  return ['rule', $rule_value];
-}
-
-sub opt_group {
-  my $ast = shift;
-  # blank group would return true :{ () }
-  return ['true'] if $ast eq '()';
-  my $atoms = opt_token_atoms($ast);
-  if (in(first($atoms), ['lbranch', 'branch'])) {
-    return $atoms;
-  }
-  return ['group', $atoms];
-}
-
-sub opt_token {
-  my $args = shift;
-  my $atoms = opt_token_atoms($args);
-  if (in(first($atoms), ['lbranch', 'branch'])) {
-    return $atoms;
-  }
-  return first($atoms) if len($atoms) == 1;
-  return ['token', $atoms];
-}
-
-#my $data = [["assert","^"],["Group",[["rtoken","ws"],["branch","||"],["ctoken","atom"]]],["rept","+"],["assert",'$']];
-#my $opt_data = opt_token_atoms($data);
-#see $opt_data;
-#exit;
-sub opt_token_atoms {
-  my $atoms = shift;
-  # see $atoms;
-  $atoms = delete_branch($atoms);
-  # see $atoms;
-  my $lb_flag = ['lbranch','|'];
-  if (in($lb_flag, $atoms)) {
-    my $branch = gather_branch($atoms, $lb_flag);
-    # see $branch;
-    return opt_lbranch($branch);
-  }
-  my $b_flag = ['branch', '||'];
-  if (in($b_flag, $atoms)) {
-    # say 'atoms include b-flag';
-    my $branch = gather_branch($atoms, $b_flag);
-    my $opt_branch = opt_branch($branch);
-    # see $opt_branch;
-    return $opt_branch;
-  }
-  my ($index, $opt_atoms) = (0, []);
-  while ($index < len($atoms)) {
-    my $look = next_atom_is_look($atoms, $index);
-    my $rept = next_atom_is_rept($atoms, $index);
-    if ($look) {
-      # see $look;
-      push $opt_atoms, $look;
-      $index += 3;
-    } elsif ($rept) {
-      push $opt_atoms, $rept;
-      $index += 2;
-    } else {
-      my $atom = $atoms->[$index];
-      push $opt_atoms, opt_atom($atom);
-      $index += 1;
-    }
-  }
-  return $opt_atoms;
-}
-
 sub delete_branch {
   my $ast = shift;
   my $ast_len = len($ast);
@@ -359,8 +120,229 @@ sub gather_branch {
   return $opt_atoms;
 }
 
+sub opt_atoms {
+  my $atoms = shift;
+  my $atoms_opt = [];
+  for my $atom (values $atoms) {
+    push $atoms_opt, opt_atom($atom);
+  }
+  return $atoms_opt;
+}
+
+sub opt_exprs {
+  my $atoms = shift;
+  my $atoms_opt = opt_atoms($atoms);
+  return add_exprs($atoms_opt);
+}
+
+sub opt_atom {
+  my $atom = shift;
+  return ['bool','true'] if $atom eq '{}';
+  # see $atom;
+  my ($type, $value) = @{$atom};
+  # see $name; exit;
+  return opt_exprs($atom) if is_perl_array($type);
+  given ($type) {
+    when ('sym')     { opt_sym($value)    }
+    when ('char')    { opt_char($value)   }
+    when ('Str')     { opt_str($value)    }
+    when ('Keyword') { opt_str($value)    }
+    when ('String')  { opt_string($value) }
+    when ('Sarray')  { opt_sarray($value) }
+    when ('List')    { opt_list($value)   }
+    when ('Array')   { opt_array($value)  }
+    when ('Hash')    { opt_hash($value)   }
+    when ('Pair')    { opt_atoms($value)  }
+    when ('Rule')    { opt_rule($value)   }
+    when ('Group')   { opt_group($value)  }
+    when ('Strs')    { opt_strs($value)   }
+    when ('Chclass') { opt_chclass($value)}
+    when ('Action')  { opt_atom($value)   }
+    when ('Alias')   { opt_alias($value)  }
+    when ('chars')   { ['str', $value]    }
+    when ('int')     { ['int', $value+0]  }
+    when ('any')     { ['any', '.']       }
+    when ('dot')     { ['dot', '.']       }
+    when ('ctoken')  { ['ctoken', $value] }
+    when ('rtoken')  { ['rtoken', $value] }
+    when ('gtoken')  { ['gtoken', $value] }
+    when ('assert')  { ['assert', $value] }
+    when ('cclass')  { ['cclass', $value] }
+    # if String only have blank interlatation
+    when ('inter')   { ['str', '']        }
+    see $atom;
+    default { error("Unknown type: $type to optimizer") }
+  }
+}
+
+sub opt_sym {
+  my ($value, $at) = @_;
+  given ($value) {
+    when ('nil')   { return ['nil', 'nil']   }
+    when ('true')  { return ['bool', 'true']  }
+    when ('false') { return ['bool', 'false'] }
+    default        { return ['sym', $value]   }
+  }
+}
+
+sub opt_char {
+  my ($value) = @_;
+  my $opt_char = opt_str_char($value);
+  return ['str', $opt_char];
+}
+
+sub opt_str {
+  my ($nodes) = @_;
+  return ['str', ''] if $nodes eq "''";
+  my $chars = [];
+  for my $node (values $nodes) {
+    my ($type, $value) = @{$node};
+    given ($type) {
+      when ('char') { push $chars, opt_str_char($value) }
+      default { push $chars, $value }
+    }
+  }
+  return ['str', perl_join($chars)];
+}
+
+sub opt_string {
+  my ($ast) = @_;
+  return '' if $ast eq '""';
+  my $nodes = [];
+  for my $node (values $ast) {
+    my ($type, $value) = @{$node};
+    given ($type) {
+      when ('char') {
+        my $char_str = opt_str_char($value);
+        push $nodes, ['str', $char_str];
+      }
+      when ('dstr') { push $nodes, ['str', $value] }
+      default { push $nodes, opt_atom($node) }
+    }
+  }
+  my $str_nodes = combin_str_nodes($nodes);
+  if (len($str_nodes) == 1 and is_str(first($nodes)) ) {
+    return first($str_nodes);
+  }
+  return ['string', $str_nodes];
+}
+
+sub opt_sarray {
+  my $ast = shift;
+  my $strs = [];
+  my $str = '';
+  for my $node (values $ast) {
+    my ($type, $value) = @{ $node };
+    given ($type) {
+      when ('cstr') { $str = $str . $value }
+      when ('char') { $str = $str . opt_str_char($value) }
+      when ('blank') {
+        push $strs, ['str', $str] if len($str) > 0;
+        $str = '';
+      }
+      default { error("Unknown Aarray node type: $type") }
+    }
+  }
+  push $strs, ['str', $str] if len($str) > 0;
+  return ['array', $strs];
+}
+
+sub opt_hash {
+  my ($hash) = @_;
+  # hash would saved as array to contain keys value
+  return ['hash', []] if $hash eq '{}';
+  my $hash_opt = [];
+  for my $pair (values $hash) {
+    if (first($pair) eq 'Pair') {
+      my $pair_value = tail($pair);
+      my $opt_pair = opt_atoms($pair_value);
+      push $hash_opt, $opt_pair;
+    } else {
+      error("Hash key name not is <Pair>");
+    }
+  }
+  return ['hash', $hash_opt];
+}
+
+sub opt_list {
+  my ($atoms) = @_;
+  return ['list', []] if $atoms eq '()';
+  my $atoms_opt = opt_atoms($atoms);
+  return ['list', $atoms_opt];
+}
+
+sub opt_array {
+  my ($atoms) = @_;
+  return ['array', []] if $atoms eq '[]';
+  my $atoms_opt = opt_atoms($atoms);
+  return ['array', $atoms_opt];
+}
+
+sub opt_rule {
+  my ($atoms) = @_;
+  return ['bool','true'] if $atoms eq ':{}';
+  my $rule_value = opt_token($atoms);
+  return ['rule', $rule_value];
+}
+
+sub opt_group {
+  my ($ast) = @_;
+  return ['bool','true'] if $ast eq '()';
+  my $atoms = opt_token_atoms($ast);
+  if (in(first($atoms), ['lbranch', 'branch'])) {
+    return $atoms;
+  }
+  return ['group', $atoms];
+}
+
+sub opt_token {
+  my $args = shift;
+  my $atoms = opt_token_atoms($args);
+  if (in(first($atoms), ['lbranch', 'branch'])) {
+    return $atoms;
+  }
+  return first($atoms) if len($atoms) == 1;
+  return ['token', $atoms];
+}
+
+#my $data = [["any","."],["look","*?"]];
+#see opt_token_atoms($data);
+#exit;
+sub opt_token_atoms {
+  my $atoms = shift;
+  $atoms = delete_branch($atoms);
+  my $lb_flag = ['lbranch','|'];
+  if (in($lb_flag, $atoms)) {
+    my $branch = gather_branch($atoms, $lb_flag);
+    return opt_lbranch($branch);
+  }
+  my $b_flag = ['branch', '||'];
+  if (in($b_flag, $atoms)) {
+    my $branch = gather_branch($atoms, $b_flag);
+    my $opt_branch = opt_branch($branch);
+    return $opt_branch;
+  }
+  my ($index, $opt_atoms) = (0, []);
+  while ($index < len($atoms)) {
+    my $look = next_atom_is_look($atoms, $index);
+    my $rept = next_atom_is_rept($atoms, $index);
+    if ($look) {
+      push $opt_atoms, $look;
+      $index += 3;
+    } elsif ($rept) {
+      push $opt_atoms, $rept;
+      $index += 2;
+    } else {
+      my $atom = $atoms->[$index];
+      push $opt_atoms, opt_atom($atom);
+      $index += 1;
+    }
+  }
+  return $opt_atoms;
+}
+
 sub opt_lbranch {
-   my $ast   = shift;
+   my ($ast) = @_;
    my $atoms = [];
    for my $branch (values $ast) {
      push $atoms, opt_token($branch);
@@ -369,7 +351,7 @@ sub opt_lbranch {
 }
 
 sub opt_branch {
-   my $ast     = shift;
+   my ($ast) = @_;
    my $atoms   = [];
    for my $branch (values $ast) {
      push $atoms, opt_token($branch);
@@ -397,6 +379,9 @@ sub next_atom_is_look {
    my $next_atom = $atoms->[$index+1];
    return 0 if first($next_atom) ne 'look';
    my $first_atom = $atoms->[$index];
+   if (($index + 2) >= len($atoms)) {
+     error("look atom missed");
+   }
    my $look_atom = $atoms->[ $index + 2 ];
    my $atom_opt = opt_atom($first_atom);
    my $rept_opt = opt_rept($next_atom->[1]);
@@ -416,7 +401,7 @@ sub opt_strs {
   my $str_list = [];
   foreach my $node (values $ast) {
     if ( is_str($node) ) {
-      push $str_list, $node;
+      push $str_list, value($node);
     }
   }
   return ['strs', $str_list];
@@ -425,11 +410,11 @@ sub opt_strs {
 sub opt_rept {
    my $str = shift;
    given ($str) {
-     when ('?')  { return [ 0,  1, $str ] }
-     when ('*')  { return [ 0, -1, $str ] }
-     when ('+')  { return [ 1, -1, $str ] }
-     when ('*?') { return [ 0, -1, $str ] }
-     when ('+?') { return [ 1, -1, $str ] }
+     when ('?')  { [ 0,  1, $str ] }
+     when ('*')  { [ 0, -1, $str ] }
+     when ('+')  { [ 1, -1, $str ] }
+     when ('*?') { [ 0, -1, $str ] }
+     when ('+?') { [ 1, -1, $str ] }
      default { error("Unkown Rept value: $str") }
    }
 }
@@ -445,14 +430,14 @@ sub opt_chclass {
 
 sub opt_chclass_node {
   my $node = shift;
-  my ($name, $value) = @{$node};
-  given ($name) {
-    when ('flip')   { return ['flip'] }
-    when ('cchar')  { return $value }
-    when ('char')   { return $value }
+  my ($type, $value) = @{ $node };
+  given ($type) {
+    when ('flip')   { return ['flip', 1] }
+    when ('cchar')  { return ['char', $value] }
+    when ('char')   { return ['char', $value] }
     when ('cclass') { return ['cclass', $value] }
     when ('range')  { return ['range', $value] }
-    default { error("Unknown char class node: $name") }
+    default { error("Unknown char class node: $type") }
   }
 }
 
