@@ -10,7 +10,7 @@ Version 0.01
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 =head1 SYNOPSIS
 
@@ -21,24 +21,12 @@ Tools gather some small reused function
     my $first_element = first([1,2,3]);
     # 1
 
-=head1 EXPORT
-
-add_exprs all_is_array all_is_int all_is_str
-all_is_sym apply_char array_index bool error fill_array
-first get_atoms_type get_token_name in is_array
-is_match_atom is_match_atoms is_case is_dot is_else is_elsif is_fail
-is_false is_hash is_if is_in is_int is_lambda is_list
-is_match is_nil is_perl_array is_func is_perl_hash
-is_ref is_rule is_same is_str is_sym is_context is_true 
-is_when len load_file perl_join perl_max perl_split perl_zip read_file
-rest second see subarray tail to_str trim type uuid value
-write_file
-
 =cut
 
 use Exporter;
 our @ISA = qw(Exporter);
 our @EXPORT = qw(
+create_cursor
 add_exprs
 all_is_array
 all_is_int
@@ -46,7 +34,6 @@ all_is_str
 all_is_sym
 apply_char
 array_index
-bool
 error
 fill_array
 first
@@ -54,6 +41,7 @@ get_atoms_type
 get_token_name
 in
 is_array
+is_bool
 is_case
 is_context
 is_dot
@@ -74,6 +62,8 @@ is_match_atoms
 is_nil
 is_perl_array
 is_perl_hash
+is_perl_int
+is_perl_str
 is_ref
 is_rule
 is_same
@@ -84,13 +74,11 @@ is_when
 len
 load_file
 perl_join
-perl_max
 perl_split
 perl_substr
 perl_zip
 read_file
 rest
-second
 see
 subarray
 tail
@@ -100,6 +88,11 @@ type
 uuid
 value
 write_file
+name_match
+gather_match
+get_rule_file
+perl_sum
+perl_concat
 );
 
 use 5.020;
@@ -107,14 +100,24 @@ use Carp qw(croak);
 use JSON qw(encode_json decode_json);
 use List::Util qw(max);
 use experimental qw(switch autoderef);
-use List::MoreUtils qw(pairwise firstidx);
+use List::MoreUtils qw(pairwise);
 
 ###################################################
-#see add_exprs([1,2,3]);
-#see add_exprs([[[1,2,3]]]);
+
+sub create_cursor {
+  my $str = shift;
+  my $trim_str = trim($str);
+  return {
+    STR  => $trim_str,
+    POS  => 0,
+    LEN  => len($trim_str),
+    LOG  => [],
+  };
+}
+
 sub add_exprs {
   my $atoms = shift;
-  return first($atoms) if len($atoms) == 1;
+  return $atoms->[0] if len($atoms) == 1;
   return ['exprs', $atoms];
 }
 
@@ -153,13 +156,10 @@ sub apply_char {
 
 sub array_index {
   my ($value, $array) = @_;
-  return firstidx { is_same($_, $value) } @{$array};
-}
-
-sub bool {
-  my $x = shift;
-  return ['true'] if $x;
-  return ['false'];
+  for my $key (keys $array) {
+    return $key if is_same($array->[$key], $value);
+  }
+  return '';
 }
 
 sub error { say @_; exit }
@@ -176,19 +176,16 @@ sub fill_array {
 sub first {
   my $data = shift;
   return $data->[0] if is_perl_array($data);
-  return substr($data, 0, 1) if is_str($data);
+  return substr($data, 0, 1) if is_perl_str($data);
 }
 
 sub get_atoms_type {
   my $atoms = shift;
   my $type = type(first($atoms));
-  my $types = [];
   for my $atom (values $atoms) {
-    my $atom_type = type($atom);
-    next if $atom_type eq $type;
-    push $types, $atom_type;
+    next if type($atom) eq $type;
+    return 0
   }
-  return $types if len($types) > 0;
   return $type;
 }
 
@@ -203,7 +200,7 @@ sub get_token_name {
 sub in {
   my ($element, $array) = @_;
   my $element_str = to_str($element);
-  for my $x (@{$array}) {
+  for my $x (values $array) {
     return 1 if to_str($x) eq $element_str;
   }
   return 0;
@@ -215,9 +212,15 @@ sub is_array {
   return 0;
 }
 
+sub is_bool {
+  my $atom = shift;
+  return 1 if type($atom) eq 'bool';
+  return 0;
+}
+
 sub is_case {
   my $x = shift;
-  return 1 if is_sym($x) and value($x) eq 'case';
+  return 1 if is_sym($x) and $x->[1] eq 'case';
   return 0;
 }
 
@@ -235,13 +238,13 @@ sub is_dot {
 
 sub is_else {
   my $x = shift;
-  return 1 if is_sym($x) and value($x) eq 'else';
+  return 1 if is_sym($x) and $x->[1] eq 'else';
   return 0;
 }
 
 sub is_elsif {
   my $x = shift;
-  return 1 if is_sym($x) and value($x) eq 'elsif';
+  return 1 if is_sym($x) and $x->[1] eq 'elsif';
   return 0;
 }
 
@@ -253,7 +256,7 @@ sub is_fail {
 
 sub is_false {
   my $x = shift;
-  return 1 if type($x) eq 'false';
+  return 1 if is_bool($x) and $x->[1] eq 'false';
   return 0;
 }
 
@@ -271,23 +274,19 @@ sub is_hash {
 
 sub is_if {
   my $x = shift;
-  return 1 if is_sym($x) and value($x) eq 'if';
+  return 1 if is_sym($x) and $x->[1] eq 'if';
   return 0;
 }
 
 sub is_in {
   my $x = shift;
-  return 1 if is_sym($x) and value($x) eq 'in';
+  return 1 if is_sym($x) and $x->[1] eq 'in';
   return 0;
 }
 
 sub is_int {
   my $x = shift;
-  if (is_str($x)) {
-    return 0 if $x ^ $x;
-    return 0 if $x eq '';
-    return 1;
-  }
+  return 1 if type($x) eq 'int';
   return 0;
 }
 
@@ -305,8 +304,8 @@ sub is_list {
 
 sub is_match {
   my $x = shift;
-  return 1 if is_true($x) or is_str($x);
-  return 0 if is_fail($x);
+  return 0 if is_false($x);
+  return 1 if is_true($x) or is_perl_str($x);
   return 1 if is_match_atom($x) or is_match_atoms($x);
   my $data_str = to_str($x);
   error("error match data: $data_str");
@@ -316,7 +315,7 @@ sub is_match_atom {
    my $x = shift;
    return 0 unless is_perl_array($x);
    return 0 unless len($x) > 1;
-   return 0 unless is_str(first($x));
+   return 0 unless is_perl_str($x->[0]);
    return 1;
 }
 
@@ -348,6 +347,22 @@ sub is_perl_hash {
    return 0;
 }
 
+sub is_perl_int {
+  my $x = shift;
+  if (is_perl_str($x)) {
+    return 0 if $x ^ $x;
+    return 0 if $x eq '';
+    return 1;
+  }
+  return 0;
+}
+
+sub is_perl_str {
+   my $x = shift;
+   return 1 if ref($x) eq ref('');
+   return 0;
+}
+
 sub is_ref {
   my $x = shift;
   return 1 if type($x) eq 'ref';
@@ -361,18 +376,15 @@ sub is_rule {
 }
 
 sub is_same {
-  my ($data_one, $data_two) = @_;
-  if (is_str($data_one) and is_str($data_two)) {
-    return ($data_one eq $data_two);
-  }
-  return 1 if to_str($data_one) eq to_str($data_two);
+  my ($data_a, $data_b) = @_;
+  return 1 if to_str($data_a) eq to_str($data_b);
   return 0;
 }
 
 sub is_str {
-   my $x = shift;
-   return 1 if ref($x) eq ref('');
-   return 0;
+  my $x = shift;
+  return 1 if type($x) eq 'str';
+  return 0;
 }
 
 sub is_sym {
@@ -383,21 +395,21 @@ sub is_sym {
 
 sub is_true {
   my $x = shift;
-  return 1 if type($x) eq 'true';
+  return 1 if is_bool($x) and $x->[1] eq 'true';
   return 0;
 }
 
 sub is_when {
   my $x = shift;
-  return 1 if is_sym($x) and value($x) eq 'when';
+  return 1 if is_sym($x) and $x->[1] eq 'when';
   return 0;
 }
 
 sub len {
   my $data = shift;
   return scalar( @{$data} ) if is_perl_array($data);
-  return $data if is_int($data);
-  return length( $data ) if is_str($data);
+  return $data if is_perl_int($data);
+  return length( $data ) if is_perl_str($data);
 }
 
 sub load_file {
@@ -414,11 +426,6 @@ sub perl_join {
   return join('', @$array);
 }
 
-sub perl_max {
-  my $args = shift;
-  return max(@{$args});
-}
-
 sub perl_split {
   my ($str, $sep) = @_;
   if (defined $sep) {
@@ -428,7 +435,7 @@ sub perl_split {
 }
 
 sub perl_substr {
-  my ($str,$from,$len) = @_;
+  my ($str, $from, $len) = @_;
   return substr($str, $from, $len);
 }
 
@@ -439,6 +446,7 @@ sub perl_zip {
 
 sub read_file {
   my $file = shift;
+  # say $file;
   local $/;
   open my ($fh), '<', $file or die $!;
   return <$fh>;
@@ -450,20 +458,14 @@ sub rest {
    if (is_perl_array($data)) {
      return [ splice( [ @{$data} ], 1, $len_data ) ];
    }
-   return substr($data, 1) if is_str($data);
+   return substr($data, 1) if is_perl_str($data);
    error("rest only could implement with str or array");
 }
 
-sub second {
-  my $data = shift;
-  return $data->[1] if is_perl_array($data);
-  return substr($data, 1, 1) if is_str($data);
-}
-
 sub see {
   my $data = shift;
   say to_str($data);
-  return ['true'];
+  return 1;
 }
 
 sub subarray {
@@ -484,18 +486,18 @@ sub subarray {
 sub tail {
   my $data = shift;
   return $data->[-1] if is_perl_array($data);
-  return substr($data, -1) if is_str($data);
+  return substr($data, -1) if is_perl_str($data);
 }
 
 sub to_str {
   my $data = shift;
-  return $data if is_str($data);
+  return $data if is_perl_str($data);
   return encode_json($data);
 }
 
 sub trim {
   my $str = shift;
-  if (is_str($str)) {
+  if (is_perl_str($str)) {
     $str =~ s/^\s+|\s+$//g;
     return $str;
   }
@@ -505,19 +507,20 @@ sub trim {
 
 sub type {
   my $x = shift;
-  return first($x) if is_perl_array($x);
-  return 'int' if is_int($x);
-  return 'str' if is_str($x);
+  return $x->[0] if is_perl_array($x);
+  return 'int' if is_perl_int($x);
+  return 'str' if is_perl_str($x);
   my $x_str = to_str($x);
   error("Could not get $x_str type");
 }
 
 sub uuid { return scalar(rand()) }
-
+# str -> str int -> int array -> array hash -> hash
+# if is array and type is defined, is type or is array
 sub value {
   my $x = shift;
   return $x->[1] if is_perl_array($x);
-  return $x if is_str($x);
+  return $x if is_perl_str($x);
   my $x_str = to_str($x);
   error("Could not get $x_str value");
 }
@@ -526,7 +529,68 @@ sub write_file {
   my ($file, $str) = @_;
   open my ($fh), '>', $file or die $!;
   print {$fh} $str;
-  return ['true'];
+  return 1;
+}
+
+sub name_match {
+  my ( $name, $match ) = @_;
+  return $match       if is_false($match);
+  return $match       if is_true($match);
+  return [$name, $match] if is_perl_str($match);
+  return $match          if $name =~ /^[a-z_]/;
+  return [$name, [$match]] if is_match_atom($match);
+  return [$name, $match];
+}
+
+sub gather_match {
+  my ( $gather, $match ) = @_;
+  return $match if is_false($match);
+  return $match if is_true($gather);
+  if ( is_perl_str($gather) ) {
+    return $gather if is_true($match);
+    return $gather . $match if is_perl_str($match);
+    return $match if is_match_atom($match);
+    return $match if is_match_atoms($match);
+  }
+  return $gather if is_true($match) or is_perl_str($match);
+  if ( is_match_atom($gather) ) {
+    return [ $gather, $match ] if is_match_atom($match); 
+    return [ $gather, @{$match} ] if is_match_atoms($match); 
+  }
+  if ( is_match_atoms($gather) ) {
+    return [ @{$gather}, $match ] if is_match_atom($match);
+    return [ @{$gather}, @{$match} ] if is_match_atoms($match);
+  }
+  my $gather_str = to_str($gather);
+  my $match_str = to_str($match);
+  error("$gather_str could not gather match with $match_str");
+}
+
+sub get_rule_file {
+  my $name = shift;
+  for my $dir (@INC) {
+    my $path = "$dir/rule/$name.json";
+    return $path if (-e $path);
+  }
+  error("Could not get rule file: $name.json");
+}
+
+sub perl_sum {
+  my $nums = shift;
+  my $sum = 0;
+  for my $value (values $nums) {
+    $sum += $value;
+  }
+  return $sum;
+}
+
+sub perl_concat {
+  my $arrays = shift;
+  my $concat_array = [];
+  for my $array (values $arrays) {
+    push $concat_array, @{$array};
+  }
+  return $concat_array;
 }
 
 =head1 AUTHOR
