@@ -10,7 +10,7 @@ Version 0.01
 
 =cut
 
-our $VERSION = '0.01';
+our $VERSION = '0.03';
 
 
 =head1 SYNOPSIS
@@ -43,23 +43,19 @@ use experimental qw(switch autoderef);
 
 use Spp::Tools;
 
-############################################
-# Perl value to Spp value
-############################################
-
 sub value_to_atom {
   my $value = shift;
-  return $value   if is_int($value);
-  return $value   if is_str($value);
+  return ['int', $value] if is_perl_int($value);
+  return ['str', $value] if is_perl_str($value);
   return array_to_atom($value) if is_perl_array($value);
   return hash_to_atom($value)  if is_perl_hash($value);
-  croak "Have not implement $value to Spp data";
+  error("Have not implement $value to Spp data");
 }
 
 sub values_to_atom {
   my $values = shift;
   my $atoms = [];
-  for my $value (@{$values}) {
+  for my $value (values $values) {
     push $atoms, value_to_atom($value);
   }
   return $atoms;
@@ -74,8 +70,9 @@ sub array_to_atom {
 sub hash_to_atom {
   my $hash_value = shift;
   my $hash = [];
-  for (my ($key, $value) = each %{$hash_value}) {
-    push $hash, [ value_to_atom($key), value_to_atom($value) ];
+  for my $key (keys $hash_value) {
+    my $value = $hash_value->{$key};
+    push $hash, [['str', $key], value_to_atom($value)];
   }
   return ['hash', $hash];
 }
