@@ -1,6 +1,6 @@
 use 5.020;
 
-use lib 'D:/Spp/perl-spp/lib/';
+use lib 'D:/Spp/perl-spp/lib';
 use Spp qw(repl spp);
 
 my $spp_file = $ARGV[0];
